@@ -112,19 +112,65 @@ static PyObject *py_cleanup(PyObject *self, PyObject *args, PyObject *kwargs)
    Py_RETURN_NONE;
 }
 
-// python function setup(channel, direction, pull_up_down=PUD_OFF, initial=None)
+// python function setup(channel(s), direction, pull_up_down=PUD_OFF, initial=None)
 static PyObject *py_setup_channel(PyObject *self, PyObject *args, PyObject *kwargs)
 {
    unsigned int gpio;
-   int channel, direction;
+   int channel = -1;
+   int direction;
+   int i, chancount;
+   PyObject *chanlist = NULL;
+   PyObject *chantuple = NULL;
+   PyObject *tempobj;
    int pud = PUD_OFF + PY_PUD_CONST_OFFSET;
    int initial = -1;
    static char *kwlist[] = {"channel", "direction", "pull_up_down", "initial", NULL};
    int func;
 
-   if (!PyArg_ParseTupleAndKeywords(args, kwargs, "ii|ii", kwlist, &channel, &direction, &pud, &initial))
+   int setup_one(void) {
+      if (get_gpio_number(channel, &gpio))
+         return 0;
+
+      func = gpio_function(gpio);
+      if (gpio_warnings &&                             // warnings enabled and
+          ((func != 0 && func != 1) ||                 // (already one of the alt functions or
+          (gpio_direction[gpio] == -1 && func == 1)))  // already an output not set from this program)
+      {
+         PyErr_WarnEx(NULL, "This channel is already in use, continuing anyway.  Use GPIO.setwarnings(False) to disable warnings.", 1);
+      }
+
+      if (direction == OUTPUT && (initial == LOW || initial == HIGH)) {
+         output_gpio(gpio, initial);
+      }
+      setup_gpio(gpio, direction, pud);
+      gpio_direction[gpio] = direction;
+      return 1;
+   }
+
+   if (!PyArg_ParseTupleAndKeywords(args, kwargs, "Oi|ii", kwlist, &chanlist, &direction, &pud, &initial))
       return NULL;
 
+#if PY_MAJOR_VERSION > 2
+   if (PyLong_Check(chanlist)) {
+      channel = (int)PyLong_AsLong(chanlist);
+#else
+   if (PyInt_Check(chanlist)) {
+      channel = (int)PyInt_AsLong(chanlist);
+#endif
+      if (PyErr_Occurred())
+         return NULL;
+      chanlist = NULL;
+   } else if PyList_Check(chanlist) {
+      // do nothing
+   } else if PyTuple_Check(chanlist) {
+      chantuple = chanlist;
+      chanlist = NULL;
+   } else {
+      // raise exception
+       PyErr_SetString(PyExc_ValueError, "Channel must be an integer or list/tuple of integers");
+       return NULL;
+   }
+
    // check module has been imported cleanly
    if (setup_error)
    {
@@ -132,68 +178,214 @@ static PyObject *py_setup_channel(PyObject *self, PyObject *args, PyObject *kwar
       return NULL;
    }
 
-   if (get_gpio_number(channel, &gpio))
+   if (mmap_gpio_mem())
       return NULL;
 
-   if (direction != INPUT && direction != OUTPUT)
-   {
+   if (direction != INPUT && direction != OUTPUT) {
       PyErr_SetString(PyExc_ValueError, "An invalid direction was passed to setup()");
-      return NULL;
+      return 0;
    }
 
    if (direction == OUTPUT)
       pud = PUD_OFF + PY_PUD_CONST_OFFSET;
 
    pud -= PY_PUD_CONST_OFFSET;
-   if (pud != PUD_OFF && pud != PUD_DOWN && pud != PUD_UP)
-   {
+   if (pud != PUD_OFF && pud != PUD_DOWN && pud != PUD_UP) {
       PyErr_SetString(PyExc_ValueError, "Invalid value for pull_up_down - should be either PUD_OFF, PUD_UP or PUD_DOWN");
       return NULL;
    }
 
-   if (mmap_gpio_mem())
-      return NULL;
-
-   func = gpio_function(gpio);
-   if (gpio_warnings &&                             // warnings enabled and
-       ((func != 0 && func != 1) ||                 // (already one of the alt functions or
-       (gpio_direction[gpio] == -1 && func == 1)))  // already an output not set from this program)
-   {
-      PyErr_WarnEx(NULL, "This channel is already in use, continuing anyway.  Use GPIO.setwarnings(False) to disable warnings.", 1);
+   if (chanlist) {
+       chancount = PyList_Size(chanlist);
+   } else if (chantuple) {
+       chancount = PyTuple_Size(chantuple);
+   } else {
+       if (!setup_one())
+          return NULL;
+       Py_RETURN_NONE;
    }
 
-   if (direction == OUTPUT && (initial == LOW || initial == HIGH))
-   {
-      output_gpio(gpio, initial);
+   for (i=0; i<chancount; i++) {
+      if (chanlist) {
+         if ((tempobj = PyList_GetItem(chanlist, i)) == NULL) {
+            return NULL;
+         }
+      } else { // assume chantuple
+         if ((tempobj = PyTuple_GetItem(chantuple, i)) == NULL) {
+            return NULL;
+         }
+      }
+
+#if PY_MAJOR_VERSION > 2
+      if (PyLong_Check(tempobj)) {
+         channel = (int)PyLong_AsLong(tempobj);
+#else
+      if (PyInt_Check(tempobj)) {
+         channel = (int)PyInt_AsLong(tempobj);
+#endif
+         if (PyErr_Occurred())
+             return NULL;
+      } else {
+          PyErr_SetString(PyExc_ValueError, "Channel must be an integer");
+          return NULL;
+      }
+
+      if (!setup_one())
+         return NULL;
    }
-   setup_gpio(gpio, direction, pud);
-   gpio_direction[gpio] = direction;
 
    Py_RETURN_NONE;
 }
 
-// python function output(channel, value)
+// python function output(channel(s), value(s))
 static PyObject *py_output_gpio(PyObject *self, PyObject *args)
 {
    unsigned int gpio;
-   int channel, value;
+   int channel = -1;
+   int value = -1;
+   int i;
+   PyObject *chanlist = NULL;
+   PyObject *valuelist = NULL;
+   PyObject *chantuple = NULL;
+   PyObject *valuetuple = NULL;
+   PyObject *tempobj = NULL;
+   int chancount = -1;
+   int valuecount = -1;
 
-   if (!PyArg_ParseTuple(args, "ii", &channel, &value))
-      return NULL;
+   int output(void) {
+      if (get_gpio_number(channel, &gpio))
+          return 0;
 
-   if (get_gpio_number(channel, &gpio))
-       return NULL;
+      if (gpio_direction[gpio] != OUTPUT)
+      {
+         PyErr_SetString(PyExc_RuntimeError, "The GPIO channel has not been set up as an OUTPUT");
+         return 0;
+      }
 
-   if (gpio_direction[gpio] != OUTPUT)
-   {
-      PyErr_SetString(PyExc_RuntimeError, "The GPIO channel has not been set up as an OUTPUT");
-      return NULL;
+      if (check_gpio_priv())
+         return 0;
+
+      output_gpio(gpio, value);
+      return 1;
    }
 
-   if (check_gpio_priv())
-      return NULL;
+   if (!PyArg_ParseTuple(args, "OO", &chanlist, &valuelist))
+       return NULL;
+
+#if PY_MAJOR_VERSION >= 3
+   if (PyLong_Check(chanlist)) {
+      channel = (int)PyLong_AsLong(chanlist);
+#else
+   if (PyInt_Check(chanlist)) {
+      channel = (int)PyInt_AsLong(chanlist);
+#endif
+      if (PyErr_Occurred())
+         return NULL;
+      chanlist = NULL;
+   } else if (PyList_Check(chanlist)) {
+      // do nothing
+   } else if (PyTuple_Check(chanlist)) {
+      chantuple = chanlist;
+      chanlist = NULL;
+   } else {
+       PyErr_SetString(PyExc_ValueError, "Channel must be an integer or list/tuple of integers");
+       return NULL;
+   }
+
+#if PY_MAJOR_VERSION >= 3
+   if (PyLong_Check(valuelist)) {
+       value = (int)PyLong_AsLong(valuelist);
+#else
+   if (PyInt_Check(valuelist)) {
+       value = (int)PyInt_AsLong(valuelist);
+#endif
+      if (PyErr_Occurred())
+         return NULL;
+       valuelist = NULL;
+   } else if (PyList_Check(valuelist)) {
+      // do nothing
+   } else if (PyTuple_Check(valuelist)) {
+      valuetuple = valuelist;
+      valuelist = NULL;
+   } else {
+       PyErr_SetString(PyExc_ValueError, "Value must be an integer/boolean or a list/tuple of integers/booleans");
+       return NULL;
+   }
+
+   if (chanlist)
+       chancount = PyList_Size(chanlist);
+   if (chantuple)
+       chancount = PyTuple_Size(chantuple);
+   if (valuelist)
+       valuecount = PyList_Size(valuelist);
+   if (valuetuple)
+       valuecount = PyTuple_Size(valuetuple);
+   if ((chancount != -1 && chancount != valuecount && valuecount != -1) || (chancount == -1 && valuecount != -1)) {
+       PyErr_SetString(PyExc_RuntimeError, "Number of channels != number of values");
+       return NULL;
+   }
+
+   if (chancount == -1) {
+      if (!output())
+         return NULL;
+      Py_RETURN_NONE;
+   }
+
+   for (i=0; i<chancount; i++) {
+      // get channel number
+      if (chanlist) {
+         if ((tempobj = PyList_GetItem(chanlist, i)) == NULL) {
+            return NULL;
+         }
+      } else { // assume chantuple
+         if ((tempobj = PyTuple_GetItem(chantuple, i)) == NULL) {
+            return NULL;
+         }
+      }
+
+#if PY_MAJOR_VERSION >= 3
+      if (PyLong_Check(tempobj)) {
+         channel = (int)PyLong_AsLong(tempobj);
+#else
+      if (PyInt_Check(tempobj)) {
+         channel = (int)PyInt_AsLong(tempobj);
+#endif
+         if (PyErr_Occurred())
+             return NULL;
+      } else {
+          PyErr_SetString(PyExc_ValueError, "Channel must be an integer");
+          return NULL;
+      }
+
+      // get value
+      if (valuecount > 0) {
+          if (valuelist) {
+             if ((tempobj = PyList_GetItem(valuelist, i)) == NULL) {
+                return NULL;
+             }
+          } else { // assume valuetuple
+             if ((tempobj = PyTuple_GetItem(valuetuple, i)) == NULL) {
+                return NULL;
+             }
+          }
+#if PY_MAJOR_VERSION >= 3
+          if (PyLong_Check(tempobj)) {
+             value = (int)PyLong_AsLong(tempobj);
+#else
+          if (PyInt_Check(tempobj)) {
+             value = (int)PyInt_AsLong(tempobj);
+#endif
+             if (PyErr_Occurred())
+                 return NULL;
+          } else {
+              PyErr_SetString(PyExc_ValueError, "Value must be an integer or boolean");
+              return NULL;
+          }
+      }
+      if (!output())
+         return NULL;
+   }
 
-   output_gpio(gpio, value);
    Py_RETURN_NONE;
 }
 
@@ -534,7 +726,7 @@ static PyObject *py_wait_for_edge(PyObject *self, PyObject *args, PyObject *kwar
       Py_INCREF(Py_None);
       return Py_None;
    } else if (result == 1) {
-      PyErr_SetString(PyExc_RuntimeError, "Conflicting edge detection events already exists for this GPIO channel");
+      PyErr_SetString(PyExc_RuntimeError, "Conflicting edge detection events already exist for this GPIO channel");
       return NULL;
    } else {
       PyErr_SetString(PyExc_RuntimeError, "Error waiting for edge");
@@ -637,9 +829,9 @@ static PyObject *py_setwarnings(PyObject *self, PyObject *args)
 static const char moduledocstring[] = "GPIO functionality of a Raspberry Pi using Python";
 
 PyMethodDef rpi_gpio_methods[] = {
-   {"setup", (PyCFunction)py_setup_channel, METH_VARARGS | METH_KEYWORDS, "Set up the GPIO channel, direction and (optional) pull/up down control\nchannel        - either board pin number or BCM number depending on which mode is set.\ndirection      - INPUT or OUTPUT\n[pull_up_down] - PUD_OFF (default), PUD_UP or PUD_DOWN\n[initial]      - Initial value for an output channel"},
+   {"setup", (PyCFunction)py_setup_channel, METH_VARARGS | METH_KEYWORDS, "Set up a GPIO channel or list of channels with a direction and (optional) pull/up down control\nchannel        - either board pin number or BCM number depending on which mode is set.\ndirection      - INPUT or OUTPUT\n[pull_up_down] - PUD_OFF (default), PUD_UP or PUD_DOWN\n[initial]      - Initial value for an output channel"},
    {"cleanup", (PyCFunction)py_cleanup, METH_VARARGS | METH_KEYWORDS, "Clean up by resetting all GPIO channels that have been used by this program to INPUT with no pullup/pulldown and no event detection\n[channel] - individual channel to clean up.  Default - clean every channel that has been used."},
-   {"output", py_output_gpio, METH_VARARGS, "Output to a GPIO channel\nchannel - either board pin number or BCM number depending on which mode is set.\nvalue   - 0/1 or False/True or LOW/HIGH"},
+   {"output", py_output_gpio, METH_VARARGS, "Output to a GPIO channel or list of channels\nchannel - either board pin number or BCM number depending on which mode is set.\nvalue   - 0/1 or False/True or LOW/HIGH"},
    {"input", py_input_gpio, METH_VARARGS, "Input from a GPIO channel.  Returns HIGH=1=True or LOW=0=False\nchannel - either board pin number or BCM number depending on which mode is set."},
    {"setmode", py_setmode, METH_VARARGS, "Set up numbering mode to use for channels.\nBOARD - Use Raspberry Pi board numbers\nBCM   - Use Broadcom GPIO 00..nn numbers"},
    {"add_event_detect", (PyCFunction)py_add_event_detect, METH_VARARGS | METH_KEYWORDS, "Enable edge detection events for a particular GPIO channel.\nchannel      - either board pin number or BCM number depending on which mode is set.\nedge         - RISING, FALLING or BOTH\n[callback]   - A callback function for the event (optional)\n[bouncetime] - Switch bounce timeout in ms for callback"},
@@ -699,7 +891,7 @@ PyMODINIT_FUNC initGPIO(void)
       pin_to_gpio = &pin_to_gpio_rev1;
    } else if (revision == 2) {
       pin_to_gpio = &pin_to_gpio_rev2;
-   } else { // assume model B+
+   } else { // assume model B+ or A+
       pin_to_gpio = &pin_to_gpio_rev3;
    }
 
